use yansi::{Color, Paint};

use crate::{expect_yes, install::uncomment_tag, linux::is_root, linux::run_command, print_status};

/// Build the `kxkbrc` file for the given `layout` and `variant
pub fn build_kxkbrc(layout: &str, variant: Option<&str>) -> String {
    let mut res = String::from("[Layout]\nUse=true\n");
    res.push_str(&format!("LayoutList={layout}\n"));
    if let Some(variant) = variant {
        res.push_str(&format!("VariantList={variant}\n"));
    }

    res
}

pub fn create_iso(
    without_gui: bool,
    no_tmp: bool,
    kb_layout: &str,
    kb_variant: Option<&str>,
    install: Option<&String>,
) {
    if !is_root() {
        eprintln!("Error: You need root to create an ISO");
        std::process::exit(1);
    }

    if install.is_some() {
        println!(
            "{} The --install option will create an ISO file which immidiately upon boot installs the selected config.",
            "WARNING:".paint(Color::Yellow)
        );
        print!("{}", "DISCLAIMER: I know that this could be dangerous and I myself am ultimately responsible for any damages [yes/no]: ".paint(Color::Red));
        expect_yes();
    }

    if !std::fs::exists("./iso").unwrap() {
        let cmd = vec!["git", "clone", "https://git.hydrar.de/navos/iso"];
        run_command(&cmd, None, false);
    }

    if without_gui || install.is_some() {
        std::fs::remove_file("./iso/airootfs/etc/systemd/system/display-manager.service").unwrap();
    } else {
        print_status("Adding GUI packages");
        uncomment_tag("#gui: ", "./iso/packages.x86_64");
    }

    if let Some(install_conf) = install {
        setup_auto_install(&install_conf);
    }

    print_status("Setting keyboard layout");
    std::fs::create_dir_all("./iso/airootfs/etc/skel/.config").unwrap();
    std::fs::write(
        "./iso/airootfs/etc/skel/.config/kxkbrc",
        build_kxkbrc(kb_layout, kb_variant),
    )
    .unwrap();

    std::fs::create_dir_all("./work").unwrap();

    if !no_tmp {
        let mount_cmd = vec!["mount", "-t", "tmpfs", "-o", "size=10G", "tmpfs", "./work"];
        run_command(&mount_cmd, None, false);
    }

    let mkarchiso_cmd = vec!["mkarchiso", "-v", "-w", "./work", "-o", "./", "./iso"];

    run_command(&mkarchiso_cmd, None, true);

    let umount_cmd = vec!["umount", "-r", "./work"];

    run_command(&umount_cmd, None, false);

    std::fs::remove_dir_all("./work").unwrap();
}

pub fn setup_auto_install(conf: &str) {
    print_status("Setting up automatic install upon boot");
    std::fs::write(
        "./iso/airootfs/etc/motd",
        "Welcome to navOS!\n\nThe system will now install itself.\n",
    )
    .unwrap();
    std::fs::copy(conf, "./iso/airootfs/root/config.toml").unwrap();
    std::fs::write(
        "./iso/airootfs/root/.zshrc",
        "navinstall install --force /root/config.toml;sleep 30;reboot\n",
    )
    .unwrap();
}
